#!/bin/sh

# Copyright 2004-2008 Proyecto Brutalix
# Liberado bajo licencia GPL <http://www.gnu.org/licenses/gpl.txt>

# Ejecutables necesarios: 
#    gzip, gunzip, pv, nc, dialog, ntfsclone, udpcast, sfdisk

DIALOG=dialog
BACKTITLE="Clonaciones Brutales"
MYIP=$(/sbin/ifconfig | grep Bcast | awk '{print $2}' | cut -d':' -f2)
MYMAC=$(/sbin/ifconfig | grep HWaddr | awk '{print $5}')
MYAULA=$(cat /proc/cmdline | awk ' BEGIN { FS = "aula=" } ; { print $2 }' |  awk '{ print $1 }')
BACKTITLE="${BACKTITLE}      IP=${MYIP}    MAC=${MYMAC}    AULA=${MYAULA}"
PORTUNI=1234
PORTMULTI=1235
TMPDIR=/tmp

d_msgbox() {
  $DIALOG --backtitle "$BACKTITLE" --msgbox "$1" 5 50
}

d_inputbox() {
  $DIALOG --backtitle "$BACKTITLE" --inputbox "$1" 10 50 \
    2> $tempfile
  return $?
}

d_menu() {
  title="$1"
  shift
  $DIALOG --backtitle "$BACKTITLE" --menu "$title" 15 60 10 "$@" \
    2> $tempfile
  return $?
}

choose_device() {
  dev_devices=$(ls /dev/[hs]d?[1-9] | cut -d'/' -f3)
  devices=$(for i in $dev_devices; do echo -n "$i /dev/$i "; done)
  d_menu "Dispositivo:" $devices

}

cliente_unicast() {
  choose_device
  retval=$?
  device=$(cat $tempfile)
  if [ $retval -eq 0 ]; then
   size_b=$(nc -l -q 1 -p $PORTUNI )
   ( nc -l -q 5 -p $PORTUNI | gunzip | pv -n -s $size_b | ntfsclone -r -O /dev/$device - 2>&1 > /dev/null ) 2>&1 | dialog --backtitle "$BACKTITLE" --gauge 'Recibiendo imagen...' 7 70
   if [ $? -eq 0 ]; then
     d_msgbox "Equipo clonado con exito"
   else
     d_msgbox "Cancelado por el usuario"
   fi
  else
    d_msgbox "Cancelado por el usuario"
  fi
}

cliente_multicast() {
  choose_device 
  retval=$?
  device=$(cat $tempfile)
  if [ $retval -eq 0 ]; then
    dialog --infobox "Esperando al servidor..." 5 50
    size_b=$( udp-receiver --portbase $PORTMULTI 2>&- )
    ( udp-receiver --portbase $PORTMULTI 2>&- | pv -n -s $size_b | ntfsclone -r -O /dev/$device - 1>/dev/null ) 2>&1 | dialog --backtitle "$BACKTITLE" --gauge 'Recibiendo imagen...' 7 70
  else
    d_msgbox "Cancelado por el usuario"
  fi
}

cliente_p2p() {
  d_msgbox "Cliente P2P"
}

servidor_unicast() {
  choose_device
  retval=$?
  device=$(cat $tempfile)
  if [ $retval -eq 0 ]; then
    d_inputbox "IP cliente:" 
    retval=$?
    ip=$(cat $tempfile)
    if [ $retval -eq 0 ]; then
      d_msgbox "Espere a que el cliente este preparado" 
      size_mb=$( ntfsresize -niPb /dev/$device | grep ^Space | awk '{print $5}')
      let size_b=$size_mb*1000*1000
      echo $size_b | nc -q 1 $ip $PORTUNI
      ( ntfsclone -s -o - /dev/$device 2>&- | pv -n -s $size_b | gzip -c | nc -q 5 $ip $PORTUNI ) 2>&1 | dialog --backtitle "$BACKTITLE" --gauge 'Enviando imagen...' 7 70
    else
      d_msgbox "Cancelado por el usuario"
    fi
  else
      d_msgbox "Cancelado por el usuario"
  fi
}

servidor_multicast() {
  choose_device 
  retval=$?
  device=$(cat $tempfile)
  if [ $retval -eq 0 ]; then
    size_mb=$( ntfsresize -niPb /dev/$device | grep ^Space | awk '{print $5}')
    let size_b=$size_mb*1000*1000
    d_msgbox "Espere a los clientes"
    echo $size_b | udp-sender --autostart 1 --portbase $PORTMULTI 2>&-
    (ntfsclone -s -o - /dev/$device 2>&- | pv -n -s $size_b | udp-sender --autostart 1 --portbase  $PORTMULTI 2>/dev/null ) 2>&1 | dialog --backtitle "$BACKTITLE" --gauge 'Enviando imagen...' 7 70
  else
    d_msgbox "Cancelado por el usuario"
  fi
}

servidor_p2p() {
  d_msgbox "Servidor P2P"
}

cliente() {
  d_menu "Modo:" \
    unicast "Cliente unico" multicast "Varios clientes sincronizados" p2p "Varios clientes P2P" 
  retval=$?
  modo=$(cat $tempfile)
  if [ $retval -eq 0 ]; then
    if [ "$modo" = "unicast" ]; then
      cliente_unicast
    elif [ "$modo" = "multicast" ]; then
      cliente_multicast
    else
      cliente_p2p
    fi
  else 
    d_msgbox "Cancelado por el usuario"
  fi
}

servidor() {
  d_menu "Modo:" \
    unicast "Cliente unico" multicast "Varios clientes sincronizados" p2p "Varios clientes P2P"
  retval=$?
  modo=$(cat $tempfile)
  if [ $retval -eq 0 ]; then
    if [ "$modo" = "unicast" ]; then
      servidor_unicast
    elif [ "$modo" = "multicast" ]; then
      servidor_multicast
    else
      servidor_p2p
    fi
  else
    d_msgbox "Cancelado por el usuario"
  fi
}

gescen() {
  d_msgbox "Gestion centralizada no implementada aun"
}

utilidades() {
  d_menu "Utilidades:" \
    1 "Guardar imagen en disco" 2 "Guardar MBR en disco" 3 "Restaurar imagen de disco" 4 "Restaurar MBR de disco" 
  retval=$?
  util=$(cat $tempfile)
  if [ $retval -eq 0 ]; then
    if [ "$util" = "1" ]; then
      echo Guardar imagen en disco
    elif [ "$util" = "2" ]; then
      echo Guardar MBR en disco
    elif [ "$util" = "3" ]; then
      echo Restaurar imagen de disco
    else
      echo Restaurar MBR de disco
    fi
  else
    d_msgbox "Cancelado por el usuario"
  fi
}

## MAIN ##

tempfile=$(tempfile 2>/dev/null) || tempfile=${TMPDIR}/file$$
trap "rm -f $tempfile" 0 1 2 5 15

d_menu "Menu Principal" \
  Cliente "Recibir imagen"  Servidor "Enviar imagen" "Gestion centralizada" "Servidor central" Utilidades Otros 
retval=$?
rol=$(cat $tempfile)
if [ $retval -eq 0 ]; then
  if [ "$rol" = "Cliente" ]; then
    cliente
  elif [ "$rol" = "Servidor" ]; then
    servidor
  elif [ "$rol" = "Gestin centralizada" ]; then
    gescen
  else
    utilidades
  fi
else
  d_msgbox "Cancelado por el usuario"
fi

exit 0
